@@ -219,6 +219,7 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
+    # TODO: unify with download_process_results (see download_apply_model_results)
     def download_descriptor_results(self,process_id,wait=False,wait_interval=3.,output_path=None):
         """Get status or results of a descriptor process.
 
@@ -320,7 +321,7 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def new_dataset(self,filepath,name,description=''):
+    def new_dataset(self,filepath,name,description='',table_name='',table_desc=''):
         """Create a new dataset on Aionics.
 
         The data for the dataset should be in .csv format.
@@ -331,11 +332,18 @@ class Client(object):
         Parameters
         ----------
         filepath : str
-            Path to dataset csv file on the local filesystem.
+            Path to a csv file on the local filesystem-
+            this file defines the dataset's first table.
         name : str
             Name for the new dataset- must be unique.
         description : str
             Description of the new dataset.
+        table_name : str
+            Name for the dataset's first table-
+            defaults to the dataset name if not provided 
+        table_desc : str
+            Description for the dataset's first table- 
+            defaults to the dataset description if not provided 
 
         Returns
         -------
@@ -346,7 +354,12 @@ class Client(object):
             self.service_url+'api/new_dataset',
             files={
                 'dataset':open(filepath,'rb'),
-                'metadata':json.dumps({'name':name,'description':description}).encode()
+                'metadata':json.dumps({
+                    'name':name,
+                    'description':description,
+                    'table_name':table_name,
+                    'table_description':table_desc,
+                }).encode()
             }
         )
         if resp.status_code == 200:
@@ -388,17 +401,102 @@ class Client(object):
         -------
         response : dict 
             Response data or error report, 
-            including the dataset (as a pandas.DataFrame) if successful.
+            including the dataset table (as a pandas.DataFrame) if successful.
         """
-        resp = self.session.get(
-            self.service_url+'api/get_dataset',
-            params={'dataset_id':dataset_id}
+        msg = 'Deprecation: get_dataset(dataset_id) is deprecated. '\
+            'Use get_table(table_id) instead. '\
+            'Use get_table_index(dataset_id) to get an index of tables '\
+            'that are attached to the dataset.'\
+            'Use get_table_id(table_name) to get the id of a table by name.'
+        raise RuntimeError(msg)
+
+    def get_table_index(self,dataset_id):
+        resp = self.session.get(self.service_url+'api/table_index',json={'dataset_id':dataset_id})
+        if resp.status_code == 200:
+            return resp.json()
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+    def new_table(self,dataset_id,filepath,name,description=''):
+        """Create a new table under a dataset on Aionics.
+
+        The data for the table should be in .csv format.
+        The .csv should include a header for column names.
+        One column should be named "id", and should contain unique integers-
+        if it is not present, it is added automatically after the upload.
+
+        Parameters
+        ----------
+        dataset_id : int
+            Id of the dataset to associate with the new table.
+        filepath : str
+            Path to csv file containing table data on the local filesystem.
+        name : str
+            Name for the new table- must be unique. 
+        description : str
+            Description of the new table (optional).
+
+        Returns
+        -------
+        response : dict
+            Dict containing the new table id or an error report.
+        """
+        resp = self.session.post(
+            self.service_url+'api/new_table',
+            files={
+                'table':open(filepath,'rb'),
+                'metadata':json.dumps({
+                    'dataset_id':dataset_id,
+                    'name':name,
+                    'description':description
+                }).encode()
+            }
         )
         if resp.status_code == 200:
-            return {'success':True,'dataset':pd.read_csv(StringIO(resp._content.decode()),index_col='id')}
+            return resp.json()
         else:
-            respdata = resp.json()
-            return {'success':False,'status_code':resp.status_code,'error':respdata['error']}
+            return {'success':False,'status_code':resp.status_code}
+
+    def get_table(self,table_id):
+        resp = self.session.get(
+            self.service_url+'api/download_table',
+            params={'table_id':table_id}
+        )
+        if resp.status_code == 200:
+            return {
+                'success':True,
+                'table':pd.read_csv(
+                    StringIO(resp._content.decode()),
+                    index_col='id'
+                )
+            }
+        else:
+            return {
+                'success':False,
+                'status_code':resp.status_code
+            }
+
+    def get_table_id(self,table_name):
+        """Get the id of a table by providing its name.
+
+        Parameters
+        ----------
+        table_name : str 
+            Name of the desired table. 
+
+        Returns
+        -------
+        response : dict 
+            Response data or error report. 
+        """
+        resp = self.session.get(
+            self.service_url+'api/get_table_id',
+            params={'table_name':table_name}
+        )
+        if resp.status_code == 200:
+            return resp.json()
+        else:
+            return {'success':False,'status_code':resp.status_code}
 
     def delete_dataset(self,dataset_id):
         """Delete a dataset from Aionics, permanently.
@@ -454,44 +552,23 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def copy_dataset(self,dataset_id,name):
-        """Copy an existing dataset to a new dataset.
+
+    # MODELS
+
+    def get_model_index(self,dataset_id):
+        """Download the index of available models for a given dataset.
 
         Parameters
         ----------
         dataset_id : int
-            Integer id of the dataset to be copied.
-        name : str
-            New name for the copied dataset.
-            Dataset names must be unique across all datasets.
+            Id of the dataset for which the model index will be downloaded.
 
         Returns
         -------
         resp : dict
-            Contains status report and, if successful, 
-            the dataset_id of the new dataset.
+            Response containing a name-to-id map for models under the specified dataset.
         """
-        resp = self.session.post(
-            self.service_url+'api/copy_dataset', 
-            json={'dataset_id':dataset_id,'name':name}
-        )
-        if resp.status_code == 200:
-            return resp.json()
-        else:
-            return {'success':False,'status_code':resp.status_code}
-
-
-    # MODELS
-
-    def get_model_index(self):
-        """Download the index of available models.
-
-        Returns
-        -------
-        resp : dict
-            Contains information about models available to the client.
-        """
-        resp = self.session.get(self.service_url+'api/model_index')
+        resp = self.session.get(self.service_url+'api/model_index',json={'dataset_id':dataset_id})
         if resp.status_code == 200:
             return resp.json()
         else:
@@ -519,13 +596,13 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def new_model(self,dataset_id,name,description=''):
+    def new_model(self,table_id,name,description=''):
         """Create a new model on Aionics.
 
         Parameters
         ----------
-        dataset_id : int 
-            Id of the dataset that is to be used for this model.
+        table_id : int 
+            Id of the table that will be the model's training set. 
         name : str
             Name for the new model- must be unique across all models.
         description : str
@@ -539,7 +616,7 @@ class Client(object):
         resp = self.session.post(
             self.service_url+'api/new_model',
             json={
-                'dataset_id':dataset_id,
+                'table_id':table_id,
                 'name':name,
                 'description':description
             }
@@ -671,15 +748,20 @@ class Client(object):
 
     # DESIGNS
 
-    def get_design_index(self):
-        """Download the index of available designs.
+    def get_design_index(self,dataset_id):
+        """Download the index of available designs for a given dataset.
+
+        Parameters
+        ----------
+        dataset_id : int
+            Id of the dataset for which the design index will be downloaded. 
 
         Returns
         -------
         resp : dict
-            Contains information about designs available to the client.
+            Response data including a name-to-id map for designs under the specified dataset. 
         """
-        resp = self.session.get(self.service_url+'api/design_index')
+        resp = self.session.get(self.service_url+'api/design_index',json={'dataset_id':dataset_id})
         if resp.status_code == 200:
             return resp.json()
         else:
@@ -707,13 +789,13 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def new_design(self,dataset_id,name,description=''):
+    def new_design(self,table_id,name,description=''):
         """Create a new design on Aionics.
 
         Parameters
         ----------
-        dataset_id : int 
-            Id of the dataset that is to be used for this design.
+        table_id : int 
+            Id of the table that will be the design's training set. 
         name : str
             Name for the new design- must be unique.
         description : str
@@ -727,7 +809,7 @@ class Client(object):
         resp = self.session.post(
             self.service_url+'api/new_design',
             json={
-                'dataset_id':dataset_id,
+                'table_id':table_id,
                 'name':name,
                 'description':description
             }
@@ -1014,12 +1096,14 @@ class Client(object):
             - 'inspection_model' (str): Options:
 
                 - 'ridge': Ridge regression- only for 'rfe_coefs' strategy
-                - 'logistic': Logistic regression- only for 'combinatorial' strategy
+                - 'logistic': Logistic regression- works for 'rfe_coefs' or 'combinatorial' strategies
  
             - 'regularization' (float): Regularization coefficient for 'ridge' model
             - 'output_encoding' (str): Output encoding for 'logistic' model- either 'categorical' or 'scalar_threshold'
             - 'output_threshold' (float): Output threshold for 'logistic' model with 'scalar_threshold' encoding- values greater than or equal to the output threshold are flagged True, and the others are flagged False.
-            - 'penalty' (str): Specification of penalty function- for 'logistic' model- either 'none', 'l1', or 'l2'
+            - 'balance_class_weights' (bool): Whether or not to weight samples by class population when evaluating the objective for fitting a classifier 
+            - 'multiclass_averaging' (str): Determines how classifier performance metrics are averaged over classes- either 'weighted', 'macro', or 'micro'
+            - 'penalty' (str): Specification of penalty function for 'logistic' model- either 'none', 'l1', or 'l2'
             - 'cost' (float): Inverse regularization coefficient for 'logistic' model
             - 'min_feats' (int): Minimum number of features to inspect- only for 'combinatorial' strategy
             - 'max_feats' (int): Maximum number of features to inspect- for 'combinatorial', 'rfe_coefs', and 'variances'
@@ -1113,6 +1197,8 @@ class Client(object):
             - 'regularization' (float): Regularization coefficient for 'ridge' model
             - 'output_encoding' (str): Output encoding for 'logistic' model- either 'categorical' or 'scalar_threshold'
             - 'output_threshold' (float): Output threshold for 'logistic' model with 'scalar_threshold' encoding- values greater than or equal to the output threshold are flagged True, and the others are flagged False.
+            - 'balance_class_weights' (bool): Whether or not to weight samples by class population when evaluating the objective for fitting a classifier 
+            - 'multiclass_averaging' (str): Determines how classifier performance metrics are averaged over classes- either 'weighted', 'macro', or 'micro'
             - 'penalty' (str): Specification of penalty function- for 'logistic' model- either 'none', 'l1', or 'l2'
             - 'cost' (float): Inverse regularization coefficient for 'logistic' model
             - 'kernel' (str): Kernel selection for 'gp' model- either 'rbf' or 'inv_exp' 
@@ -1263,7 +1349,6 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    # TODO: DEPRECATE
     def download_apply_model_results(self,model_id,process_id,wait=False,wait_interval=1.,output_path=None):
         """Fetch results or progress report for a model execution.
 
@@ -1294,7 +1379,7 @@ class Client(object):
             output_path=output_path
         )
         # COMPATIBILITY PATCH
-        res['df'] = res.pop('data')
+        res['df'] = res.get('data')
         return res
 
     def download_process_results(self,process_id,wait=False,wait_interval=1.,make_dataframe=False,output_path=''):
@@ -1404,11 +1489,12 @@ class Client(object):
         settings : dict
             Dict of general candidate configurations:
 
-            - 'candidate_source' (str): Specifies source of candidates. Either 'candidate_generator' (default) or 'dataset'.
-            - 'candidate_dataset_id' (int): If 'candidate_source'=='dataset', this is the integer id of the dataset that defines the candidates.
+            - 'candidate_source' (str): Specifies source of candidates. Either 'candidate_generator' (default) or 'table'.
+            - 'candidate_table_id' (int): If 'candidate_source'=='table', this is the integer id of the table that defines the candidates.
 
         substance_settings : dict
-            Settings for generating candidate substances.
+            Settings for generating candidate substances 
+            (only used if settings['candidate_source']=='candidate_generator').
             The keys of this dict should be the names of the substance columns.
             The values are dicts of settings for generating candidates for that substance. 
             Each dict may have the following keys:
@@ -1422,7 +1508,8 @@ class Client(object):
             - 'conc_resolution' (float): If 'conc_source'=='grid', this is the grid spacing.
 
         scalar_settings : dict
-            Settings for generating candidate scalar values.
+            Settings for generating candidate scalar values
+            (only used if settings['candidate_source']=='candidate_generator').
             The keys of this dict should be the names of the scalar columns.
             The values are dicts of settings for generating candidate scalar values. 
             Each dict may have the following keys:
@@ -1478,7 +1565,9 @@ class Client(object):
                 - 'ignore' : Input space screening is skipped.
                 - 'distance' : Candidates are scored based on their Euclidean distance from training set samples in the model training space. For explorative screening with little or no output data.
             
-            - 'input_space_sequential' (bool): Whether or not to screen the input space sequentially. If True, each candidate is evaluated against the training set and all of the candidates that were selected before it. If False, the candidates are all evaluated equally against only the training set.
+            - 'input_space_target' (str): target for input space evaluation, either 'max' or 'min'. 
+            - 'input_space_model' (str): Name of the model to use for input space evaluation. 
+            - 'input_space_sequential' (str): Sequential evaluation mode. Either 'sequential' or 'none' (default is 'none'). 
             - 'input_space_weight' (float): For joint screening, this is the weight applied to the input space acquisition value.
 
         model_settings : dict
@@ -1947,6 +2036,114 @@ class Client(object):
             return {'success':False,'status_code':resp.status_code}
         return {'success':True,'settings':settings,'results':results}
 
+    def download_model_file(self,model_id,file_tag,output_path):
+        """Download a file for a given model by providing the file tag and model id.
+
+        Parameters
+        ----------
+        model_id : int
+            Integer index of the desired model.
+        file_tag : str
+            String indicating which file to download.
+            Depending on model settings,
+            the following file tags may exist: 
+
+            - 'SOURCE_DATASET': the model's training set table 
+            - 'CONFIGURED_DATASET': training set table post-configuration 
+            - 'OUTPUT_HISTOGRAM': graphical histogram of output values 
+            - 'FEATURIZED_DATASET': training set table post-featurization
+            - 'PCA_DATASET': training set table post-PCA, pre-standardization
+            - 'TRANSFORMED_DATASET': training set table post-transformation
+            - 'TRANSFORMED_OUTPUT_HISTOGRAM': graphical histogram of transformed output values 
+            - 'FS_OBJECTIVES': graphical plot of model performance objectives with respect to input space dimensionality 
+            - 'SELECTED_FEATURES_BARCHART': graphical bar chart of selected feature coefficients 
+            - 'FS_MODEL_PERFORMANCE': predicted (and cross-validated) versus actual values in transformed space for the model used in feature selection
+            - 'FS_MODEL_RESIDUALS_TR': graphical histogram of residuals in transformed space for the model used in feature selection
+            - 'FS_MODEL_CONFUSION_TRAINING': graphical training set confusion matrix for the model used in feature selection
+            - 'FS_MODEL_CONFUSION_XVAL': graphical cross-validation confusion matrix for the model used in feature selection
+            - 'FS_DATASET': training set table post-feature selection 
+            - 'MODEL_PERFORMANCE_TR': predicted (and cross-validated) versus actual values in transformed space for the fully-trained model
+            - 'MODEL_PERFORMANCE': predicted (and cross-validated) versus actual values for the fully-trained model
+            - 'MODEL_RESIDUALS_TR': graphical histogram of trained model residuals in transformed space
+            - 'MODEL_PREDICTION_HISTOGRAM_TR': graphical histogram of trained model predictions and cross-validations in transformed space
+            - 'MODEL_PREDICTION_HISTOGRAM': graphical histogram of trained model predictions and cross-validations
+            - 'MODEL_PREDICTION_VARIANCES_TR': graphical histogram of trained model prediction and cross-validation variances in transformed space
+            - 'MODEL_PREDICTION_VARIANCES': graphical histogram of trained model prediction and cross-validation variances
+            - 'MODEL_CONFUSION_TRAINING': graphical training set confusion matrix for the trained model
+            - 'MODEL_CONFUSION_XVAL': graphical cross-validation confusion matrix for the trained model
+            - 'TRAINED_DATASET': training set table post-model training 
+
+        output_path : str
+            Local filesystem path where the file should be saved.
+
+        Returns
+        -------
+        response : dict 
+            Response data or error report. 
+        """
+        resp = self.session.get(
+            self.service_url+'model_file',
+            params={
+                'model_id':model_id,
+                'file_tag':file_tag
+            }
+        )
+        if resp.status_code == 200:
+            output_dir, fname = os.path.split(output_path)
+            if not os.path.exists(output_dir):
+                raise IOError('output directory {} does not exist'.format(output_dir))
+            open(output_path,'wb').write(resp._content)
+            return {'success':True}
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+    def download_design_file(self,design_id,file_tag,output_path):
+        """Download a file for a given design by providing the file tag and design id.
+
+        Parameters
+        ----------
+        design_id : int
+            Integer index of the desired design.
+        file_tag : str
+            String indicating which file to download.
+            Depending on design settings,
+            the following file tags may exist: 
+
+            - 'SOURCE_DATASET': the design's main table (defines the design space)
+            - 'CANDIDATE_SOURCE_DATASET': copy of the data table defining the design candidates 
+            - 'SCREENED_CANDIDATE_DATASET': data table of candidates post-screening
+            - '<output>_SCREENING_PROB_HISTOGRAM': graphical histogram of prediction probabilities for <output>
+            - '<output>_SCREENING_PRED_HISTOGRAM': graphical histogram of predictions for <output>
+            - '<output>_SCREENING_TRANS_PRED_HISTOGRAM': graphical histogram of transformed predictions for <output>
+            - '<output>_SCREENING_VAR_HISTOGRAM': graphical histogram of prediction variances for <output>
+            - '<output>_SCREENING_TRANS_VAR_HISTOGRAM': graphical histogram of transformed prediction variances for <output>
+            - '<output>_SCREENING_TRANS_PRED_VAR_SCATTER': graphical scatter of transformed variance-versus-prediction for <output>
+            - '<output>_SCREENING_PRED_VAR_SCATTER': graphical scatter of variance-versus-prediction for <output>
+
+        output_path : str
+            Local filesystem path where the file should be saved.
+
+        Returns
+        -------
+        response : dict 
+            Response data or error report. 
+        """
+        resp = self.session.get(
+            self.service_url+'design_file',
+            params={
+                'design_id':design_id,
+                'file_tag':file_tag
+            }
+        )
+        if resp.status_code == 200:
+            output_dir, fname = os.path.split(output_path)
+            if not os.path.exists(output_dir):
+                raise IOError('output directory {} does not exist'.format(output_dir))
+            open(output_path,'wb').write(resp._content)
+            return {'success':True}
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
     def download_measurement_file(self,measurement_id,file_tag,output_path):
         """Download a measurement file by providing the file tag and measurement id.
 
@@ -1988,8 +2185,7 @@ class Client(object):
             open(output_path,'wb').write(resp._content)
             return {'success':True}
         else:
-            respdata = resp.json()
-            return {'success':False,'status_code':resp.status_code,'error':respdata['error']}
+            return {'success':False,'status_code':resp.status_code}
 
     def edit_measurement_metadata(self,measurement_id,name='',description=''):
         """Edit measurement metadata (name and description).
