@@ -101,7 +101,7 @@ class SubstancesClient(object):
                 'other_names':other_names,
                 'descriptors':descriptors,
                 'properties':properties,
-                'documents':documents
+                'documents':documents,
                 'remove_other_descriptors':remove_other_descriptors,
                 'remove_other_properties':remove_other_properties,
                 'remove_other_documents':remove_other_documents
@@ -119,15 +119,7 @@ class SubstancesClient(object):
     def check_substance_names(self,names):
         """Check for existence of substances by name.
 
-        Parameters
-        ----------
-        names : list
-            List of names (strings) to be checked.
-
-        Returns
-        -------
-        response : dict
-            Dict containing status report and results.
+        See documentation for apyonics.client.Client.check_substance_names().
         """
         resp = self.session.get(
             self.service_url+'check_substance_names', 
@@ -143,16 +135,7 @@ class SubstancesClient(object):
     def get_substance_data(self,name):
         """Get all data for a substance by providing a substance name.
 
-        Parameters
-        ----------
-        name : str
-            A name that is assigned to the substance of interest.
-
-        Returns
-        -------
-        response : dict
-            Dict containing status report and, if successful,
-            a dict of data about the substance.
+        See documentation for apyonics.client.Client.get_substance_data().
         """
         resp = self.session.get(
             self.service_url+'download_substance', 
@@ -170,28 +153,7 @@ class SubstancesClient(object):
     def get_descriptors(self,substance_names,descriptor_names=[]):
         """Get substance descriptors by providing substance names.
 
-        Descriptors are looked up and returned for all substance names
-        that are found in Aionics' substances database.
-        Substance names that are not found in the database 
-        are dropped from the table of results.
-
-        This function returns a process id that can be used
-        to track the progress of the computation and, 
-        when finished, fetch the results.
-
-        Parameters
-        ----------
-        substance_names : list
-            List of names (strings) of each substance to be looked up.
-        descriptor_names : list 
-            If provided, the returned descriptors will be filtered
-            to include only these names. 
-
-        Returns
-        -------
-        response : dict
-            Dict containing status report and, if successful,
-            a pandas DataFrame full of descriptor data. 
+        See documentation for apyonics.client.Client.get_descriptors().
         """
         resp = self.session.post(
             self.service_url+'get_descriptors', 
